# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["LogoCreateResponse"]


class LogoCreateResponse(BaseModel):
    id: Optional[int] = None

    tag: Optional[str] = None

    upload_path: Optional[str] = None



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from ...._models import BaseModel

__all__ = ["TokenRotateResponse"]


class TokenRotateResponse(BaseModel):
    token: str
    """New token value (shown once). Treat as sensitive."""

    type: Literal["publish_subscribe", "subscribe"]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .custom_csrs import (
    CustomCsrsResource,
    AsyncCustomCsrsResource,
    CustomCsrsResourceWithRawResponse,
    AsyncCustomCsrsResourceWithRawResponse,
    CustomCsrsResourceWithStreamingResponse,
    AsyncCustomCsrsResourceWithStreamingResponse,
)

__all__ = [
    "CustomCsrsResource",
    "AsyncCustomCsrsResource",
    "CustomCsrsResourceWithRawResponse",
    "AsyncCustomCsrsResourceWithRawResponse",
    "CustomCsrsResourceWithStreamingResponse",
    "AsyncCustomCsrsResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, cast

import httpx

from ..._types import Body, Query, Headers, NotGiven, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ...types.ssl import auto_origin_tls_kex_edit_params
from ..._base_client import make_request_options
from ...types.ssl.auto_origin_tls_kex_get_response import AutoOriginTLSKexGetResponse
from ...types.ssl.auto_origin_tls_kex_edit_response import AutoOriginTLSKexEditResponse

__all__ = ["AutoOriginTLSKexResource", "AsyncAutoOriginTLSKexResource"]


class AutoOriginTLSKexResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AutoOriginTLSKexResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AutoOriginTLSKexResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AutoOriginTLSKexResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AutoOriginTLSKexResourceWithStreamingResponse(self)

    def edit(
        self,
        *,
        zone_id: str,
        enabled: bool,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AutoOriginTLSKexEditResponse:
        """
        Enable or disable Auto-Origin TLS KEX selection for the zone by sending
        `{"enabled": true}` or `{"enabled": false}`. When enabled, Cloudflare runs a
        periodic scan of the zone's origins to determine the preferred key-exchange
        algorithm and writes that preference to the edge so it is sent first in the TLS
        ClientHello to the origin.

        Args:
          enabled: Controls enablement of Auto-Origin TLS KEX selection for the zone.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._patch(
            path_template("/zones/{zone_id}/settings/auto_origin_tls_kex", zone_id=zone_id),
            body=maybe_transform({"enabled": enabled}, auto_origin_tls_kex_edit_params.AutoOriginTLSKexEditParams),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[AutoOriginTLSKexEditResponse]._unwrapper,
            ),
            cast_to=cast(Type[AutoOriginTLSKexEditResponse], ResultWrapper[AutoOriginTLSKexEditResponse]),
        )

    def get(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AutoOriginTLSKexGetResponse:
        """
        When enabled, Cloudflare automatically selects the preferred TLS key-exchange
        algorithm to use when establishing the TLS connection to the zone's origin,
        picking from the algorithms permitted by the zone's
        `origin_tls_compliance_modes` setting. When disabled, the default key-exchange
        ordering is used.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return self._get(
            path_template("/zones/{zone_id}/settings/auto_origin_tls_kex", zone_id=zone_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[AutoOriginTLSKexGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[AutoOriginTLSKexGetResponse], ResultWrapper[AutoOriginTLSKexGetResponse]),
        )


class AsyncAutoOriginTLSKexResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAutoOriginTLSKexResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAutoOriginTLSKexResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAutoOriginTLSKexResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncAutoOriginTLSKexResourceWithStreamingResponse(self)

    async def edit(
        self,
        *,
        zone_id: str,
        enabled: bool,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AutoOriginTLSKexEditResponse:
        """
        Enable or disable Auto-Origin TLS KEX selection for the zone by sending
        `{"enabled": true}` or `{"enabled": false}`. When enabled, Cloudflare runs a
        periodic scan of the zone's origins to determine the preferred key-exchange
        algorithm and writes that preference to the edge so it is sent first in the TLS
        ClientHello to the origin.

        Args:
          enabled: Controls enablement of Auto-Origin TLS KEX selection for the zone.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._patch(
            path_template("/zones/{zone_id}/settings/auto_origin_tls_kex", zone_id=zone_id),
            body=await async_maybe_transform(
                {"enabled": enabled}, auto_origin_tls_kex_edit_params.AutoOriginTLSKexEditParams
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[AutoOriginTLSKexEditResponse]._unwrapper,
            ),
            cast_to=cast(Type[AutoOriginTLSKexEditResponse], ResultWrapper[AutoOriginTLSKexEditResponse]),
        )

    async def get(
        self,
        *,
        zone_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AutoOriginTLSKexGetResponse:
        """
        When enabled, Cloudflare automatically selects the preferred TLS key-exchange
        algorithm to use when establishing the TLS connection to the zone's origin,
        picking from the algorithms permitted by the zone's
        `origin_tls_compliance_modes` setting. When disabled, the default key-exchange
        ordering is used.

        Args:
          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not zone_id:
            raise ValueError(f"Expected a non-empty value for `zone_id` but received {zone_id!r}")
        return await self._get(
            path_template("/zones/{zone_id}/settings/auto_origin_tls_kex", zone_id=zone_id),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[AutoOriginTLSKexGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[AutoOriginTLSKexGetResponse], ResultWrapper[AutoOriginTLSKexGetResponse]),
        )


class AutoOriginTLSKexResourceWithRawResponse:
    def __init__(self, auto_origin_tls_kex: AutoOriginTLSKexResource) -> None:
        self._auto_origin_tls_kex = auto_origin_tls_kex

        self.edit = to_raw_response_wrapper(
            auto_origin_tls_kex.edit,
        )
        self.get = to_raw_response_wrapper(
            auto_origin_tls_kex.get,
        )


class AsyncAutoOriginTLSKexResourceWithRawResponse:
    def __init__(self, auto_origin_tls_kex: AsyncAutoOriginTLSKexResource) -> None:
        self._auto_origin_tls_kex = auto_origin_tls_kex

        self.edit = async_to_raw_response_wrapper(
            auto_origin_tls_kex.edit,
        )
        self.get = async_to_raw_response_wrapper(
            auto_origin_tls_kex.get,
        )


class AutoOriginTLSKexResourceWithStreamingResponse:
    def __init__(self, auto_origin_tls_kex: AutoOriginTLSKexResource) -> None:
        self._auto_origin_tls_kex = auto_origin_tls_kex

        self.edit = to_streamed_response_wrapper(
            auto_origin_tls_kex.edit,
        )
        self.get = to_streamed_response_wrapper(
            auto_origin_tls_kex.get,
        )


class AsyncAutoOriginTLSKexResourceWithStreamingResponse:
    def __init__(self, auto_origin_tls_kex: AsyncAutoOriginTLSKexResource) -> None:
        self._auto_origin_tls_kex = auto_origin_tls_kex

        self.edit = async_to_streamed_response_wrapper(
            auto_origin_tls_kex.edit,
        )
        self.get = async_to_streamed_response_wrapper(
            auto_origin_tls_kex.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ....._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ....._utils import path_template, maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._base_client import make_request_options
from .....types.zero_trust.casb.applications import setup_flow_list_params
from .....types.zero_trust.casb.applications.setup_flow_list_response import SetupFlowListResponse

__all__ = ["SetupFlowsResource", "AsyncSetupFlowsResource"]


class SetupFlowsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> SetupFlowsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return SetupFlowsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> SetupFlowsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return SetupFlowsResourceWithStreamingResponse(self)

    def list(
        self,
        slug: str,
        *,
        account_id: str,
        auth_method: str | Omit = omit,
        environment: Literal["fedramp", "standard"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SetupFlowListResponse:
        """
        Returns all available setup flows for the application, one per auth method.

        Args:
          auth_method: Filter by auth method slug. Get available slugs from GET /v2/applications.

          environment: Filter by environment.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not slug:
            raise ValueError(f"Expected a non-empty value for `slug` but received {slug!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/one/applications/{slug}/setup-flows", account_id=account_id, slug=slug
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "auth_method": auth_method,
                        "environment": environment,
                    },
                    setup_flow_list_params.SetupFlowListParams,
                ),
            ),
            cast_to=SetupFlowListResponse,
        )


class AsyncSetupFlowsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncSetupFlowsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncSetupFlowsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncSetupFlowsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncSetupFlowsResourceWithStreamingResponse(self)

    async def list(
        self,
        slug: str,
        *,
        account_id: str,
        auth_method: str | Omit = omit,
        environment: Literal["fedramp", "standard"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SetupFlowListResponse:
        """
        Returns all available setup flows for the application, one per auth method.

        Args:
          auth_method: Filter by auth method slug. Get available slugs from GET /v2/applications.

          environment: Filter by environment.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not slug:
            raise ValueError(f"Expected a non-empty value for `slug` but received {slug!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/one/applications/{slug}/setup-flows", account_id=account_id, slug=slug
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=await async_maybe_transform(
                    {
                        "auth_method": auth_method,
                        "environment": environment,
                    },
                    setup_flow_list_params.SetupFlowListParams,
                ),
            ),
            cast_to=SetupFlowListResponse,
        )


class SetupFlowsResourceWithRawResponse:
    def __init__(self, setup_flows: SetupFlowsResource) -> None:
        self._setup_flows = setup_flows

        self.list = to_raw_response_wrapper(
            setup_flows.list,
        )


class AsyncSetupFlowsResourceWithRawResponse:
    def __init__(self, setup_flows: AsyncSetupFlowsResource) -> None:
        self._setup_flows = setup_flows

        self.list = async_to_raw_response_wrapper(
            setup_flows.list,
        )


class SetupFlowsResourceWithStreamingResponse:
    def __init__(self, setup_flows: SetupFlowsResource) -> None:
        self._setup_flows = setup_flows

        self.list = to_streamed_response_wrapper(
            setup_flows.list,
        )


class AsyncSetupFlowsResourceWithStreamingResponse:
    def __init__(self, setup_flows: AsyncSetupFlowsResource) -> None:
        self._setup_flows = setup_flows

        self.list = async_to_streamed_response_wrapper(
            setup_flows.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["LivestreamGetLivestreamAnalyticsCompleteParams"]


class LivestreamGetLivestreamAnalyticsCompleteParams(TypedDict, total=False):
    account_id: Required[str]
    """The account identifier tag."""

    end_time: int
    """
    Specify the end time as a Unix timestamp in seconds to access the livestream
    analytics.
    """

    filters: str
    """Optional filters for livestream analytics."""

    start_time: int
    """
    Specify the start time as a Unix timestamp in seconds to access the livestream
    analytics.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Dict, List, Union, Optional
from datetime import datetime
from typing_extensions import Literal, TypeAlias

from ..._models import BaseModel

__all__ = [
    "VersionGraphResponse",
    "Graph",
    "GraphWorkflow",
    "GraphWorkflowFunctions",
    "GraphWorkflowNode",
    "GraphWorkflowNodeUnionMember0",
    "GraphWorkflowNodeUnionMember1",
    "GraphWorkflowNodeUnionMember1Config",
    "GraphWorkflowNodeUnionMember1ConfigRetries",
    "GraphWorkflowNodeUnionMember2",
    "GraphWorkflowNodeUnionMember2Options",
    "GraphWorkflowNodeUnionMember2Payload",
    "GraphWorkflowNodeUnionMember2PayloadType",
    "GraphWorkflowNodeUnionMember2PayloadUnionMember1",
    "GraphWorkflowNodeUnionMember3",
    "GraphWorkflowNodeUnionMember4",
    "GraphWorkflowNodeUnionMember5",
    "GraphWorkflowNodeUnionMember6",
    "GraphWorkflowNodeUnionMember6CatchBlock",
    "GraphWorkflowNodeUnionMember6FinallyBlock",
    "GraphWorkflowNodeUnionMember6TryBlock",
    "GraphWorkflowNodeUnionMember7",
    "GraphWorkflowNodeUnionMember8",
    "GraphWorkflowNodeUnionMember8Branch",
    "GraphWorkflowNodeUnionMember9",
    "GraphWorkflowNodeUnionMember9Branch",
    "GraphWorkflowNodeUnionMember10",
    "GraphWorkflowNodeUnionMember10Functions",
    "GraphWorkflowNodeUnionMember10Payload",
    "GraphWorkflowNodeUnionMember10PayloadType",
    "GraphWorkflowNodeUnionMember10PayloadUnionMember1",
    "GraphWorkflowNodeUnionMember11",
    "GraphWorkflowNodeUnionMember12",
    "GraphWorkflowNodeUnionMember13",
    "GraphWorkflowPayload",
    "GraphWorkflowPayloadType",
    "GraphWorkflowPayloadUnionMember1",
]


class GraphWorkflowFunctions(BaseModel):
    name: str

    nodes: List[object]
    """Child nodes (recursive)."""

    type: Literal["function_def"]


class GraphWorkflowNodeUnionMember0(BaseModel):
    duration: Union[float, str]
    """Duration as milliseconds (number) or human-readable string."""

    name: str

    type: Literal["step_sleep"]

    resolves: Optional[float] = None

    starts: Optional[float] = None


class GraphWorkflowNodeUnionMember1ConfigRetries(BaseModel):
    """Retry policy for a step."""

    backoff: Literal["constant", "linear", "exponential"]
    """Backoff strategy for step retries."""

    delay: Union[float, str]
    """Duration as milliseconds (number) or human-readable string."""

    limit: float


class GraphWorkflowNodeUnionMember1Config(BaseModel):
    """Configuration for a step (retries and timeout)."""

    retries: GraphWorkflowNodeUnionMember1ConfigRetries
    """Retry policy for a step."""

    timeout: Union[float, str]
    """Duration as milliseconds (number) or human-readable string."""


class GraphWorkflowNodeUnionMember1(BaseModel):
    config: GraphWorkflowNodeUnionMember1Config
    """Configuration for a step (retries and timeout)."""

    name: str

    nodes: List[object]
    """Child nodes (recursive)."""

    type: Literal["step_do"]

    resolves: Optional[float] = None

    starts: Optional[float] = None


class GraphWorkflowNodeUnionMember2Options(BaseModel):
    """Options for a waitForEvent step."""

    event_type: str

    timeout: Union[float, str]
    """Duration as milliseconds (number) or human-readable string."""


class GraphWorkflowNodeUnionMember2PayloadType(BaseModel):
    type: Literal["unknown"]


class GraphWorkflowNodeUnionMember2PayloadUnionMember1(BaseModel):
    fields: Dict[str, object]
    """Nested JsonShape fields (recursive structure)."""

    type: Literal["object"]


GraphWorkflowNodeUnionMember2Payload: TypeAlias = Union[
    GraphWorkflowNodeUnionMember2PayloadType, GraphWorkflowNodeUnionMember2PayloadUnionMember1
]


class GraphWorkflowNodeUnionMember2(BaseModel):
    name: str

    options: Optional[GraphWorkflowNodeUnionMember2Options] = None
    """Options for a waitForEvent step."""

    type: Literal["step_wait_for_event"]

    payload: Optional[GraphWorkflowNodeUnionMember2Payload] = None
    """Shape descriptor for JSON payloads."""

    resolves: Optional[float] = None

    starts: Optional[float] = None


class GraphWorkflowNodeUnionMember3(BaseModel):
    name: str

    timestamp: str

    type: Literal["step_sleep_until"]

    resolves: Optional[float] = None

    starts: Optional[float] = None


class GraphWorkflowNodeUnionMember4(BaseModel):
    nodes: List[object]
    """Child nodes (recursive)."""

    type: Literal["loop"]


class GraphWorkflowNodeUnionMember5(BaseModel):
    kind: Literal["all", "any", "all_settled", "race"]
    """Parallel execution strategy."""

    nodes: List[object]
    """Child nodes (recursive)."""

    type: Literal["parallel"]


class GraphWorkflowNodeUnionMember6CatchBlock(BaseModel):
    nodes: List[object]
    """Child nodes (recursive)."""

    type: Literal["block"]


class GraphWorkflowNodeUnionMember6FinallyBlock(BaseModel):
    nodes: List[object]
    """Child nodes (recursive)."""

    type: Literal["block"]


class GraphWorkflowNodeUnionMember6TryBlock(BaseModel):
    nodes: List[object]
    """Child nodes (recursive)."""

    type: Literal["block"]


class GraphWorkflowNodeUnionMember6(BaseModel):
    catch_block: Optional[GraphWorkflowNodeUnionMember6CatchBlock] = None

    finally_block: Optional[GraphWorkflowNodeUnionMember6FinallyBlock] = None

    try_block: Optional[GraphWorkflowNodeUnionMember6TryBlock] = None

    type: Literal["try"]


class GraphWorkflowNodeUnionMember7(BaseModel):
    nodes: List[object]
    """Child nodes (recursive)."""

    type: Literal["block"]


class GraphWorkflowNodeUnionMember8Branch(BaseModel):
    condition: Optional[str] = None

    nodes: List[object]
    """Child nodes (recursive)."""


class GraphWorkflowNodeUnionMember8(BaseModel):
    branches: List[GraphWorkflowNodeUnionMember8Branch]

    type: Literal["if"]


class GraphWorkflowNodeUnionMember9Branch(BaseModel):
    condition: Optional[str] = None

    nodes: List[object]
    """Child nodes (recursive)."""


class GraphWorkflowNodeUnionMember9(BaseModel):
    branches: List[GraphWorkflowNodeUnionMember9Branch]

    discriminant: str

    type: Literal["switch"]


class GraphWorkflowNodeUnionMember10Functions(BaseModel):
    name: str

    nodes: List[object]
    """Child nodes (recursive)."""

    type: Literal["function_def"]


class GraphWorkflowNodeUnionMember10PayloadType(BaseModel):
    type: Literal["unknown"]


class GraphWorkflowNodeUnionMember10PayloadUnionMember1(BaseModel):
    fields: Dict[str, object]
    """Nested JsonShape fields (recursive structure)."""

    type: Literal["object"]


GraphWorkflowNodeUnionMember10Payload: TypeAlias = Union[
    GraphWorkflowNodeUnionMember10PayloadType, GraphWorkflowNodeUnionMember10PayloadUnionMember1
]


class GraphWorkflowNodeUnionMember10(BaseModel):
    class_name: str

    functions: Dict[str, GraphWorkflowNodeUnionMember10Functions]

    nodes: List[object]
    """Child nodes (recursive)."""

    type: Literal["start"]

    payload: Optional[GraphWorkflowNodeUnionMember10Payload] = None
    """Shape descriptor for JSON payloads."""


class GraphWorkflowNodeUnionMember11(BaseModel):
    name: str

    type: Literal["function_call"]

    resolves: Optional[float] = None

    starts: Optional[float] = None


class GraphWorkflowNodeUnionMember12(BaseModel):
    name: str

    nodes: List[object]
    """Child nodes (recursive)."""

    type: Literal["function_def"]


class GraphWorkflowNodeUnionMember13(BaseModel):
    kind: Literal["break", "return"]
    """Break or return from a loop."""

    type: Literal["break"]


GraphWorkflowNode: TypeAlias = Union[
    GraphWorkflowNodeUnionMember0,
    GraphWorkflowNodeUnionMember1,
    GraphWorkflowNodeUnionMember2,
    GraphWorkflowNodeUnionMember3,
    GraphWorkflowNodeUnionMember4,
    GraphWorkflowNodeUnionMember5,
    GraphWorkflowNodeUnionMember6,
    GraphWorkflowNodeUnionMember7,
    GraphWorkflowNodeUnionMember8,
    GraphWorkflowNodeUnionMember9,
    GraphWorkflowNodeUnionMember10,
    GraphWorkflowNodeUnionMember11,
    GraphWorkflowNodeUnionMember12,
    GraphWorkflowNodeUnionMember13,
]


class GraphWorkflowPayloadType(BaseModel):
    type: Literal["unknown"]


class GraphWorkflowPayloadUnionMember1(BaseModel):
    fields: Dict[str, object]
    """Nested JsonShape fields (recursive structure)."""

    type: Literal["object"]


GraphWorkflowPayload: TypeAlias = Union[GraphWorkflowPayloadType, GraphWorkflowPayloadUnionMember1]


class GraphWorkflow(BaseModel):
    """A parsed workflow entrypoint with its step graph."""

    class_name: str

    functions: Dict[str, GraphWorkflowFunctions]

    nodes: List[GraphWorkflowNode]

    payload: Optional[GraphWorkflowPayload] = None
    """Shape descriptor for JSON payloads."""


class Graph(BaseModel):
    """Versioned workflow graph payload."""

    version: float

    workflow: GraphWorkflow
    """A parsed workflow entrypoint with its step graph."""


class VersionGraphResponse(BaseModel):
    id: str

    class_name: str

    created_on: datetime

    graph: Optional[Graph] = None
    """Versioned workflow graph payload."""

    modified_on: datetime

    workflow_id: str



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Type, Optional, cast
from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, SequenceNotStr, omit, not_given
from ..._utils import path_template, maybe_transform, async_maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ..._wrappers import ResultWrapper
from ...types.iam import oauth_client_create_params, oauth_client_update_params
from ...pagination import SyncSinglePage, AsyncSinglePage
from ..._base_client import AsyncPaginator, make_request_options
from ...types.iam.oauth_client_get_response import OAuthClientGetResponse
from ...types.iam.oauth_client_list_response import OAuthClientListResponse
from ...types.iam.oauth_client_create_response import OAuthClientCreateResponse
from ...types.iam.oauth_client_delete_response import OAuthClientDeleteResponse
from ...types.iam.oauth_client_update_response import OAuthClientUpdateResponse
from ...types.iam.oauth_client_rotate_secret_response import OAuthClientRotateSecretResponse
from ...types.iam.oauth_client_delete_rotated_secret_response import OAuthClientDeleteRotatedSecretResponse

__all__ = ["OAuthClientsResource", "AsyncOAuthClientsResource"]


class OAuthClientsResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> OAuthClientsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return OAuthClientsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> OAuthClientsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return OAuthClientsResourceWithStreamingResponse(self)

    def create(
        self,
        *,
        account_id: str,
        client_name: str,
        grant_types: List[Literal["authorization_code", "refresh_token"]],
        redirect_uris: SequenceNotStr[str],
        response_types: List[Literal["token", "id_token", "code"]],
        scopes: SequenceNotStr[str],
        token_endpoint_auth_method: Literal["none", "client_secret_basic", "client_secret_post"],
        allowed_cors_origins: SequenceNotStr[str] | Omit = omit,
        client_uri: str | Omit = omit,
        logo_uri: str | Omit = omit,
        policy_uri: str | Omit = omit,
        post_logout_redirect_uris: SequenceNotStr[str] | Omit = omit,
        tos_uri: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[OAuthClientCreateResponse]:
        """
        Create a new OAuth client for an account.

        Args:
          account_id: Account identifier tag.

          client_name: Human-readable name of the OAuth client.

          grant_types: Array of OAuth grant types the client is allowed to use. `authorization_code` is
              required; `refresh_token` may be included optionally.

          redirect_uris: Array of allowed redirect URIs for the client.

          response_types: Array of OAuth response types the client is allowed to use.

          scopes: Array of OAuth scopes the client is allowed to request. Colon-delimited scopes
              are not accepted. Dot-delimited scopes are validated against available OAuth API
              scopes; simple identity scopes are allowed. Protocol scopes `offline_access` and
              `openid` are added or removed automatically based on `grant_types` and
              `response_types`.

          token_endpoint_auth_method: The authentication method the client uses at the token endpoint.

          allowed_cors_origins: Array of allowed CORS origins.

          client_uri: URL of the home page of the client.

          logo_uri: URL of the client's logo.

          policy_uri: URL that points to a privacy policy document.

          post_logout_redirect_uris: Array of allowed post-logout redirect URIs.

          tos_uri: URL that points to a terms of service document.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/oauth_clients", account_id=account_id),
            body=maybe_transform(
                {
                    "client_name": client_name,
                    "grant_types": grant_types,
                    "redirect_uris": redirect_uris,
                    "response_types": response_types,
                    "scopes": scopes,
                    "token_endpoint_auth_method": token_endpoint_auth_method,
                    "allowed_cors_origins": allowed_cors_origins,
                    "client_uri": client_uri,
                    "logo_uri": logo_uri,
                    "policy_uri": policy_uri,
                    "post_logout_redirect_uris": post_logout_redirect_uris,
                    "tos_uri": tos_uri,
                },
                oauth_client_create_params.OAuthClientCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[OAuthClientCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[OAuthClientCreateResponse]], ResultWrapper[OAuthClientCreateResponse]),
        )

    def update(
        self,
        oauth_client_id: str,
        *,
        account_id: str,
        allowed_cors_origins: SequenceNotStr[str] | Omit = omit,
        client_name: str | Omit = omit,
        client_uri: str | Omit = omit,
        grant_types: List[Literal["authorization_code", "refresh_token"]] | Omit = omit,
        logo_uri: str | Omit = omit,
        policy_uri: str | Omit = omit,
        post_logout_redirect_uris: SequenceNotStr[str] | Omit = omit,
        redirect_uris: SequenceNotStr[str] | Omit = omit,
        response_types: List[Literal["token", "id_token", "code"]] | Omit = omit,
        scopes: SequenceNotStr[str] | Omit = omit,
        token_endpoint_auth_method: Literal["none", "client_secret_basic", "client_secret_post"] | Omit = omit,
        tos_uri: str | Omit = omit,
        visibility: Literal["public"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[OAuthClientUpdateResponse]:
        """Update an existing OAuth client.

        Only include fields you want to update.

        Args:
          account_id: Account identifier tag.

          oauth_client_id: The unique identifier for an OAuth client.

          allowed_cors_origins: Array of allowed CORS origins.

          client_name: Human-readable name of the OAuth client.

          client_uri: URL of the home page of the client.

          grant_types: Array of OAuth grant types the client is allowed to use. `authorization_code` is
              required; `refresh_token` may be included optionally.

          logo_uri: URL of the client's logo.

          policy_uri: URL that points to a privacy policy document.

          post_logout_redirect_uris: Array of allowed post-logout redirect URIs.

          redirect_uris: Array of allowed redirect URIs for the client.

          response_types: Array of OAuth response types the client is allowed to use.

          scopes: Array of OAuth scopes the client is allowed to request. Colon-delimited scopes
              are not accepted. Dot-delimited scopes are validated against available OAuth API
              scopes; simple identity scopes are allowed. Protocol scopes `offline_access` and
              `openid` are added or removed automatically based on `grant_types` and
              `response_types`.

          token_endpoint_auth_method: The authentication method the client uses at the token endpoint.

          tos_uri: URL that points to a terms of service document.

          visibility: Promote the OAuth client from private to public visibility. Only `public` is
              accepted; demotion to `private` is not supported. Promotion requires a non-empty
              client name, logo URI, verified client URI host, and at least one non-identity
              scope.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not oauth_client_id:
            raise ValueError(f"Expected a non-empty value for `oauth_client_id` but received {oauth_client_id!r}")
        return self._patch(
            path_template(
                "/accounts/{account_id}/oauth_clients/{oauth_client_id}",
                account_id=account_id,
                oauth_client_id=oauth_client_id,
            ),
            body=maybe_transform(
                {
                    "allowed_cors_origins": allowed_cors_origins,
                    "client_name": client_name,
                    "client_uri": client_uri,
                    "grant_types": grant_types,
                    "logo_uri": logo_uri,
                    "policy_uri": policy_uri,
                    "post_logout_redirect_uris": post_logout_redirect_uris,
                    "redirect_uris": redirect_uris,
                    "response_types": response_types,
                    "scopes": scopes,
                    "token_endpoint_auth_method": token_endpoint_auth_method,
                    "tos_uri": tos_uri,
                    "visibility": visibility,
                },
                oauth_client_update_params.OAuthClientUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[OAuthClientUpdateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[OAuthClientUpdateResponse]], ResultWrapper[OAuthClientUpdateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncSinglePage[OAuthClientListResponse]:
        """
        List all OAuth clients for an account.

        Args:
          account_id: Account identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/oauth_clients", account_id=account_id),
            page=SyncSinglePage[OAuthClientListResponse],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=OAuthClientListResponse,
        )

    def delete(
        self,
        oauth_client_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[OAuthClientDeleteResponse]:
        """
        Delete an OAuth client.

        Args:
          account_id: Account identifier tag.

          oauth_client_id: The unique identifier for an OAuth client.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not oauth_client_id:
            raise ValueError(f"Expected a non-empty value for `oauth_client_id` but received {oauth_client_id!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/oauth_clients/{oauth_client_id}",
                account_id=account_id,
                oauth_client_id=oauth_client_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[OAuthClientDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[OAuthClientDeleteResponse]], ResultWrapper[OAuthClientDeleteResponse]),
        )

    def delete_rotated_secret(
        self,
        oauth_client_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[OAuthClientDeleteRotatedSecretResponse]:
        """Removes the old client secret after a rotation, keeping only the new one.

        Use
        this after you have updated your client configuration to use the new secret. The
        `has_rotated_secret` field on the client indicates whether there is an old
        secret to delete.

        Args:
          account_id: Account identifier tag.

          oauth_client_id: The unique identifier for an OAuth client.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not oauth_client_id:
            raise ValueError(f"Expected a non-empty value for `oauth_client_id` but received {oauth_client_id!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/oauth_clients/{oauth_client_id}/rotate_secret",
                account_id=account_id,
                oauth_client_id=oauth_client_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[OAuthClientDeleteRotatedSecretResponse]]._unwrapper,
            ),
            cast_to=cast(
                Type[Optional[OAuthClientDeleteRotatedSecretResponse]],
                ResultWrapper[OAuthClientDeleteRotatedSecretResponse],
            ),
        )

    def get(
        self,
        oauth_client_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[OAuthClientGetResponse]:
        """
        Get details of a specific OAuth client.

        Args:
          account_id: Account identifier tag.

          oauth_client_id: The unique identifier for an OAuth client.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not oauth_client_id:
            raise ValueError(f"Expected a non-empty value for `oauth_client_id` but received {oauth_client_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/oauth_clients/{oauth_client_id}",
                account_id=account_id,
                oauth_client_id=oauth_client_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[OAuthClientGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[OAuthClientGetResponse]], ResultWrapper[OAuthClientGetResponse]),
        )

    def rotate_secret(
        self,
        oauth_client_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[OAuthClientRotateSecretResponse]:
        """
        Creates a second client secret so you can update your client configuration
        before deleting the old one. The `has_rotated_secret` field on the client will
        be set to `true`.

        Args:
          account_id: Account identifier tag.

          oauth_client_id: The unique identifier for an OAuth client.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not oauth_client_id:
            raise ValueError(f"Expected a non-empty value for `oauth_client_id` but received {oauth_client_id!r}")
        return self._post(
            path_template(
                "/accounts/{account_id}/oauth_clients/{oauth_client_id}/rotate_secret",
                account_id=account_id,
                oauth_client_id=oauth_client_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[OAuthClientRotateSecretResponse]]._unwrapper,
            ),
            cast_to=cast(
                Type[Optional[OAuthClientRotateSecretResponse]], ResultWrapper[OAuthClientRotateSecretResponse]
            ),
        )


class AsyncOAuthClientsResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncOAuthClientsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncOAuthClientsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncOAuthClientsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncOAuthClientsResourceWithStreamingResponse(self)

    async def create(
        self,
        *,
        account_id: str,
        client_name: str,
        grant_types: List[Literal["authorization_code", "refresh_token"]],
        redirect_uris: SequenceNotStr[str],
        response_types: List[Literal["token", "id_token", "code"]],
        scopes: SequenceNotStr[str],
        token_endpoint_auth_method: Literal["none", "client_secret_basic", "client_secret_post"],
        allowed_cors_origins: SequenceNotStr[str] | Omit = omit,
        client_uri: str | Omit = omit,
        logo_uri: str | Omit = omit,
        policy_uri: str | Omit = omit,
        post_logout_redirect_uris: SequenceNotStr[str] | Omit = omit,
        tos_uri: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[OAuthClientCreateResponse]:
        """
        Create a new OAuth client for an account.

        Args:
          account_id: Account identifier tag.

          client_name: Human-readable name of the OAuth client.

          grant_types: Array of OAuth grant types the client is allowed to use. `authorization_code` is
              required; `refresh_token` may be included optionally.

          redirect_uris: Array of allowed redirect URIs for the client.

          response_types: Array of OAuth response types the client is allowed to use.

          scopes: Array of OAuth scopes the client is allowed to request. Colon-delimited scopes
              are not accepted. Dot-delimited scopes are validated against available OAuth API
              scopes; simple identity scopes are allowed. Protocol scopes `offline_access` and
              `openid` are added or removed automatically based on `grant_types` and
              `response_types`.

          token_endpoint_auth_method: The authentication method the client uses at the token endpoint.

          allowed_cors_origins: Array of allowed CORS origins.

          client_uri: URL of the home page of the client.

          logo_uri: URL of the client's logo.

          policy_uri: URL that points to a privacy policy document.

          post_logout_redirect_uris: Array of allowed post-logout redirect URIs.

          tos_uri: URL that points to a terms of service document.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/oauth_clients", account_id=account_id),
            body=await async_maybe_transform(
                {
                    "client_name": client_name,
                    "grant_types": grant_types,
                    "redirect_uris": redirect_uris,
                    "response_types": response_types,
                    "scopes": scopes,
                    "token_endpoint_auth_method": token_endpoint_auth_method,
                    "allowed_cors_origins": allowed_cors_origins,
                    "client_uri": client_uri,
                    "logo_uri": logo_uri,
                    "policy_uri": policy_uri,
                    "post_logout_redirect_uris": post_logout_redirect_uris,
                    "tos_uri": tos_uri,
                },
                oauth_client_create_params.OAuthClientCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[OAuthClientCreateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[OAuthClientCreateResponse]], ResultWrapper[OAuthClientCreateResponse]),
        )

    async def update(
        self,
        oauth_client_id: str,
        *,
        account_id: str,
        allowed_cors_origins: SequenceNotStr[str] | Omit = omit,
        client_name: str | Omit = omit,
        client_uri: str | Omit = omit,
        grant_types: List[Literal["authorization_code", "refresh_token"]] | Omit = omit,
        logo_uri: str | Omit = omit,
        policy_uri: str | Omit = omit,
        post_logout_redirect_uris: SequenceNotStr[str] | Omit = omit,
        redirect_uris: SequenceNotStr[str] | Omit = omit,
        response_types: List[Literal["token", "id_token", "code"]] | Omit = omit,
        scopes: SequenceNotStr[str] | Omit = omit,
        token_endpoint_auth_method: Literal["none", "client_secret_basic", "client_secret_post"] | Omit = omit,
        tos_uri: str | Omit = omit,
        visibility: Literal["public"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[OAuthClientUpdateResponse]:
        """Update an existing OAuth client.

        Only include fields you want to update.

        Args:
          account_id: Account identifier tag.

          oauth_client_id: The unique identifier for an OAuth client.

          allowed_cors_origins: Array of allowed CORS origins.

          client_name: Human-readable name of the OAuth client.

          client_uri: URL of the home page of the client.

          grant_types: Array of OAuth grant types the client is allowed to use. `authorization_code` is
              required; `refresh_token` may be included optionally.

          logo_uri: URL of the client's logo.

          policy_uri: URL that points to a privacy policy document.

          post_logout_redirect_uris: Array of allowed post-logout redirect URIs.

          redirect_uris: Array of allowed redirect URIs for the client.

          response_types: Array of OAuth response types the client is allowed to use.

          scopes: Array of OAuth scopes the client is allowed to request. Colon-delimited scopes
              are not accepted. Dot-delimited scopes are validated against available OAuth API
              scopes; simple identity scopes are allowed. Protocol scopes `offline_access` and
              `openid` are added or removed automatically based on `grant_types` and
              `response_types`.

          token_endpoint_auth_method: The authentication method the client uses at the token endpoint.

          tos_uri: URL that points to a terms of service document.

          visibility: Promote the OAuth client from private to public visibility. Only `public` is
              accepted; demotion to `private` is not supported. Promotion requires a non-empty
              client name, logo URI, verified client URI host, and at least one non-identity
              scope.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not oauth_client_id:
            raise ValueError(f"Expected a non-empty value for `oauth_client_id` but received {oauth_client_id!r}")
        return await self._patch(
            path_template(
                "/accounts/{account_id}/oauth_clients/{oauth_client_id}",
                account_id=account_id,
                oauth_client_id=oauth_client_id,
            ),
            body=await async_maybe_transform(
                {
                    "allowed_cors_origins": allowed_cors_origins,
                    "client_name": client_name,
                    "client_uri": client_uri,
                    "grant_types": grant_types,
                    "logo_uri": logo_uri,
                    "policy_uri": policy_uri,
                    "post_logout_redirect_uris": post_logout_redirect_uris,
                    "redirect_uris": redirect_uris,
                    "response_types": response_types,
                    "scopes": scopes,
                    "token_endpoint_auth_method": token_endpoint_auth_method,
                    "tos_uri": tos_uri,
                    "visibility": visibility,
                },
                oauth_client_update_params.OAuthClientUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[OAuthClientUpdateResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[OAuthClientUpdateResponse]], ResultWrapper[OAuthClientUpdateResponse]),
        )

    def list(
        self,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[OAuthClientListResponse, AsyncSinglePage[OAuthClientListResponse]]:
        """
        List all OAuth clients for an account.

        Args:
          account_id: Account identifier tag.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/oauth_clients", account_id=account_id),
            page=AsyncSinglePage[OAuthClientListResponse],
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            model=OAuthClientListResponse,
        )

    async def delete(
        self,
        oauth_client_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[OAuthClientDeleteResponse]:
        """
        Delete an OAuth client.

        Args:
          account_id: Account identifier tag.

          oauth_client_id: The unique identifier for an OAuth client.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not oauth_client_id:
            raise ValueError(f"Expected a non-empty value for `oauth_client_id` but received {oauth_client_id!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/oauth_clients/{oauth_client_id}",
                account_id=account_id,
                oauth_client_id=oauth_client_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[OAuthClientDeleteResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[OAuthClientDeleteResponse]], ResultWrapper[OAuthClientDeleteResponse]),
        )

    async def delete_rotated_secret(
        self,
        oauth_client_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[OAuthClientDeleteRotatedSecretResponse]:
        """Removes the old client secret after a rotation, keeping only the new one.

        Use
        this after you have updated your client configuration to use the new secret. The
        `has_rotated_secret` field on the client indicates whether there is an old
        secret to delete.

        Args:
          account_id: Account identifier tag.

          oauth_client_id: The unique identifier for an OAuth client.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not oauth_client_id:
            raise ValueError(f"Expected a non-empty value for `oauth_client_id` but received {oauth_client_id!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/oauth_clients/{oauth_client_id}/rotate_secret",
                account_id=account_id,
                oauth_client_id=oauth_client_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[OAuthClientDeleteRotatedSecretResponse]]._unwrapper,
            ),
            cast_to=cast(
                Type[Optional[OAuthClientDeleteRotatedSecretResponse]],
                ResultWrapper[OAuthClientDeleteRotatedSecretResponse],
            ),
        )

    async def get(
        self,
        oauth_client_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[OAuthClientGetResponse]:
        """
        Get details of a specific OAuth client.

        Args:
          account_id: Account identifier tag.

          oauth_client_id: The unique identifier for an OAuth client.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not oauth_client_id:
            raise ValueError(f"Expected a non-empty value for `oauth_client_id` but received {oauth_client_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/oauth_clients/{oauth_client_id}",
                account_id=account_id,
                oauth_client_id=oauth_client_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[OAuthClientGetResponse]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[OAuthClientGetResponse]], ResultWrapper[OAuthClientGetResponse]),
        )

    async def rotate_secret(
        self,
        oauth_client_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[OAuthClientRotateSecretResponse]:
        """
        Creates a second client secret so you can update your client configuration
        before deleting the old one. The `has_rotated_secret` field on the client will
        be set to `true`.

        Args:
          account_id: Account identifier tag.

          oauth_client_id: The unique identifier for an OAuth client.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not oauth_client_id:
            raise ValueError(f"Expected a non-empty value for `oauth_client_id` but received {oauth_client_id!r}")
        return await self._post(
            path_template(
                "/accounts/{account_id}/oauth_clients/{oauth_client_id}/rotate_secret",
                account_id=account_id,
                oauth_client_id=oauth_client_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[OAuthClientRotateSecretResponse]]._unwrapper,
            ),
            cast_to=cast(
                Type[Optional[OAuthClientRotateSecretResponse]], ResultWrapper[OAuthClientRotateSecretResponse]
            ),
        )


class OAuthClientsResourceWithRawResponse:
    def __init__(self, oauth_clients: OAuthClientsResource) -> None:
        self._oauth_clients = oauth_clients

        self.create = to_raw_response_wrapper(
            oauth_clients.create,
        )
        self.update = to_raw_response_wrapper(
            oauth_clients.update,
        )
        self.list = to_raw_response_wrapper(
            oauth_clients.list,
        )
        self.delete = to_raw_response_wrapper(
            oauth_clients.delete,
        )
        self.delete_rotated_secret = to_raw_response_wrapper(
            oauth_clients.delete_rotated_secret,
        )
        self.get = to_raw_response_wrapper(
            oauth_clients.get,
        )
        self.rotate_secret = to_raw_response_wrapper(
            oauth_clients.rotate_secret,
        )


class AsyncOAuthClientsResourceWithRawResponse:
    def __init__(self, oauth_clients: AsyncOAuthClientsResource) -> None:
        self._oauth_clients = oauth_clients

        self.create = async_to_raw_response_wrapper(
            oauth_clients.create,
        )
        self.update = async_to_raw_response_wrapper(
            oauth_clients.update,
        )
        self.list = async_to_raw_response_wrapper(
            oauth_clients.list,
        )
        self.delete = async_to_raw_response_wrapper(
            oauth_clients.delete,
        )
        self.delete_rotated_secret = async_to_raw_response_wrapper(
            oauth_clients.delete_rotated_secret,
        )
        self.get = async_to_raw_response_wrapper(
            oauth_clients.get,
        )
        self.rotate_secret = async_to_raw_response_wrapper(
            oauth_clients.rotate_secret,
        )


class OAuthClientsResourceWithStreamingResponse:
    def __init__(self, oauth_clients: OAuthClientsResource) -> None:
        self._oauth_clients = oauth_clients

        self.create = to_streamed_response_wrapper(
            oauth_clients.create,
        )
        self.update = to_streamed_response_wrapper(
            oauth_clients.update,
        )
        self.list = to_streamed_response_wrapper(
            oauth_clients.list,
        )
        self.delete = to_streamed_response_wrapper(
            oauth_clients.delete,
        )
        self.delete_rotated_secret = to_streamed_response_wrapper(
            oauth_clients.delete_rotated_secret,
        )
        self.get = to_streamed_response_wrapper(
            oauth_clients.get,
        )
        self.rotate_secret = to_streamed_response_wrapper(
            oauth_clients.rotate_secret,
        )


class AsyncOAuthClientsResourceWithStreamingResponse:
    def __init__(self, oauth_clients: AsyncOAuthClientsResource) -> None:
        self._oauth_clients = oauth_clients

        self.create = async_to_streamed_response_wrapper(
            oauth_clients.create,
        )
        self.update = async_to_streamed_response_wrapper(
            oauth_clients.update,
        )
        self.list = async_to_streamed_response_wrapper(
            oauth_clients.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            oauth_clients.delete,
        )
        self.delete_rotated_secret = async_to_streamed_response_wrapper(
            oauth_clients.delete_rotated_secret,
        )
        self.get = async_to_streamed_response_wrapper(
            oauth_clients.get,
        )
        self.rotate_secret = async_to_streamed_response_wrapper(
            oauth_clients.rotate_secret,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Literal

import httpx

from ..._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ..._utils import path_template, maybe_transform
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ...pagination import SyncV4PagePaginationArray, AsyncV4PagePaginationArray
from ..._base_client import AsyncPaginator, make_request_options
from ...types.email_routing import account_rule_list_params
from ...types.email_routing.account_rule import AccountRule

__all__ = ["AccountRulesResource", "AsyncAccountRulesResource"]


class AccountRulesResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AccountRulesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AccountRulesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AccountRulesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AccountRulesResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        enabled: Literal[True, False] | Omit = omit,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncV4PagePaginationArray[AccountRule]:
        """
        Lists existing routing rules across all zones in the account or zone.

        Args:
          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          enabled: Filter by enabled routing rules.

          page: Page number of paginated results.

          per_page: Maximum number of results per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        return self._get_api_list(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/email/routing/rules",
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            page=SyncV4PagePaginationArray[AccountRule],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "enabled": enabled,
                        "page": page,
                        "per_page": per_page,
                    },
                    account_rule_list_params.AccountRuleListParams,
                ),
            ),
            model=AccountRule,
        )


class AsyncAccountRulesResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncAccountRulesResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncAccountRulesResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncAccountRulesResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncAccountRulesResourceWithStreamingResponse(self)

    def list(
        self,
        *,
        account_id: str | Omit = omit,
        zone_id: str | Omit = omit,
        enabled: Literal[True, False] | Omit = omit,
        page: float | Omit = omit,
        per_page: float | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[AccountRule, AsyncV4PagePaginationArray[AccountRule]]:
        """
        Lists existing routing rules across all zones in the account or zone.

        Args:
          account_id: The Account ID to use for this endpoint. Mutually exclusive with the Zone ID.

          zone_id: The Zone ID to use for this endpoint. Mutually exclusive with the Account ID.

          enabled: Filter by enabled routing rules.

          page: Page number of paginated results.

          per_page: Maximum number of results per page.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if account_id and zone_id:
            raise ValueError("You cannot provide both account_id and zone_id")

        if account_id:
            account_or_zone = "accounts"
            account_or_zone_id = account_id
        else:
            if not zone_id:
                raise ValueError("You must provide either account_id or zone_id")

            account_or_zone = "zones"
            account_or_zone_id = zone_id
        return self._get_api_list(
            path_template(
                "/{account_or_zone}/{account_or_zone_id}/email/routing/rules",
                account_or_zone=account_or_zone,
                account_or_zone_id=account_or_zone_id,
            ),
            page=AsyncV4PagePaginationArray[AccountRule],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "enabled": enabled,
                        "page": page,
                        "per_page": per_page,
                    },
                    account_rule_list_params.AccountRuleListParams,
                ),
            ),
            model=AccountRule,
        )


class AccountRulesResourceWithRawResponse:
    def __init__(self, account_rules: AccountRulesResource) -> None:
        self._account_rules = account_rules

        self.list = to_raw_response_wrapper(
            account_rules.list,
        )


class AsyncAccountRulesResourceWithRawResponse:
    def __init__(self, account_rules: AsyncAccountRulesResource) -> None:
        self._account_rules = account_rules

        self.list = async_to_raw_response_wrapper(
            account_rules.list,
        )


class AccountRulesResourceWithStreamingResponse:
    def __init__(self, account_rules: AccountRulesResource) -> None:
        self._account_rules = account_rules

        self.list = to_streamed_response_wrapper(
            account_rules.list,
        )


class AsyncAccountRulesResourceWithStreamingResponse:
    def __init__(self, account_rules: AsyncAccountRulesResource) -> None:
        self._account_rules = account_rules

        self.list = async_to_streamed_response_wrapper(
            account_rules.list,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from typing_extensions import Literal, TypeAlias

from ...._models import BaseModel

__all__ = [
    "ApplicationListResponse",
    "ApplicationListResponseItem",
    "ApplicationListResponseItemAuthMethod",
    "ApplicationListResponseItemPermission",
    "ApplicationListResponseItemUseCase",
]


class ApplicationListResponseItemAuthMethod(BaseModel):
    """Auth method summary for list endpoint."""

    display_name: str
    """Human-readable auth method name."""

    slug: str
    """Auth method identifier."""


class ApplicationListResponseItemPermission(BaseModel):
    """Permission/scope with severity for display."""

    display_name: str
    """Human-readable permission name."""

    scope: str
    """Vendor-native scope identifier."""

    severity: Literal["low", "medium", "high", "critical"]
    """Permission sensitivity level.

    - `low` - low
    - `medium` - medium
    - `high` - high
    - `critical` - critical
    """


class ApplicationListResponseItemUseCase(BaseModel):
    """Lightweight use case for list endpoint."""

    display_name: str
    """Human-readable use case name."""

    slug: str
    """Use case identifier (e.g. casb, ces)."""


class ApplicationListResponseItem(BaseModel):
    """Application item in list response."""

    auth_methods: List[ApplicationListResponseItemAuthMethod]
    """Available auth methods."""

    category: str
    """Vendor category (e.g. Productivity, AI)."""

    description: str
    """Brief description of the integration."""

    display_name: str
    """Human-readable vendor name."""

    dlp_enabled: bool
    """Whether DLP scanning is supported."""

    logo: Optional[str] = None
    """Logo path."""

    permissions: List[ApplicationListResponseItemPermission]
    """All permissions with severity."""

    slug: Literal["GITHUB", "GOOGLE_WORKSPACE", "MICROSOFT_INTERNAL", "SALESFORCE", "SLACK"]
    """Vendor identifier (e.g. microsoft_internal, google_workspace).

    - `GITHUB` - GITHUB
    - `GOOGLE_WORKSPACE` - GOOGLE_WORKSPACE
    - `MICROSOFT_INTERNAL` - MICROSOFT_INTERNAL
    - `SALESFORCE` - SALESFORCE
    - `SLACK` - SLACK
    """

    supported_environments: List[str]
    """Environments this vendor supports (standard, fedramp)."""

    use_cases: List[ApplicationListResponseItemUseCase]
    """Supported use cases."""


ApplicationListResponse: TypeAlias = List[ApplicationListResponseItem]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Type, Optional, cast

import httpx

from ....._types import Body, Query, Headers, NotGiven, not_given
from ....._utils import path_template
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._wrappers import ResultWrapper
from ....._base_client import make_request_options
from .....types.zero_trust.access.users.identity import Identity

__all__ = ["LastSeenIdentityResource", "AsyncLastSeenIdentityResource"]


class LastSeenIdentityResource(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> LastSeenIdentityResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return LastSeenIdentityResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> LastSeenIdentityResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return LastSeenIdentityResourceWithStreamingResponse(self)

    def get(
        self,
        user_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Identity]:
        """
        Get last seen identity for a single user.

        Args:
          account_id: Identifier.

          user_id: UUID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not user_id:
            raise ValueError(f"Expected a non-empty value for `user_id` but received {user_id!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/access/users/{user_id}/last_seen_identity",
                account_id=account_id,
                user_id=user_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[Identity]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[Identity]], ResultWrapper[Identity]),
        )


class AsyncLastSeenIdentityResource(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncLastSeenIdentityResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncLastSeenIdentityResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncLastSeenIdentityResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncLastSeenIdentityResourceWithStreamingResponse(self)

    async def get(
        self,
        user_id: str,
        *,
        account_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> Optional[Identity]:
        """
        Get last seen identity for a single user.

        Args:
          account_id: Identifier.

          user_id: UUID.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not user_id:
            raise ValueError(f"Expected a non-empty value for `user_id` but received {user_id!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/access/users/{user_id}/last_seen_identity",
                account_id=account_id,
                user_id=user_id,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[Optional[Identity]]._unwrapper,
            ),
            cast_to=cast(Type[Optional[Identity]], ResultWrapper[Identity]),
        )


class LastSeenIdentityResourceWithRawResponse:
    def __init__(self, last_seen_identity: LastSeenIdentityResource) -> None:
        self._last_seen_identity = last_seen_identity

        self.get = to_raw_response_wrapper(
            last_seen_identity.get,
        )


class AsyncLastSeenIdentityResourceWithRawResponse:
    def __init__(self, last_seen_identity: AsyncLastSeenIdentityResource) -> None:
        self._last_seen_identity = last_seen_identity

        self.get = async_to_raw_response_wrapper(
            last_seen_identity.get,
        )


class LastSeenIdentityResourceWithStreamingResponse:
    def __init__(self, last_seen_identity: LastSeenIdentityResource) -> None:
        self._last_seen_identity = last_seen_identity

        self.get = to_streamed_response_wrapper(
            last_seen_identity.get,
        )


class AsyncLastSeenIdentityResourceWithStreamingResponse:
    def __init__(self, last_seen_identity: AsyncLastSeenIdentityResource) -> None:
        self._last_seen_identity = last_seen_identity

        self.get = async_to_streamed_response_wrapper(
            last_seen_identity.get,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["PipelineListV1Params"]


class PipelineListV1Params(TypedDict, total=False):
    account_id: Required[str]
    """Specifies the public ID of the account."""

    name: str
    """Filters pipelines by name (case-insensitive substring)."""

    page: float

    per_page: float



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Dict, Type, Union, Iterable, Optional, cast
from typing_extensions import Literal

import httpx

from .changelog import (
    ChangelogResource,
    AsyncChangelogResource,
    ChangelogResourceWithRawResponse,
    AsyncChangelogResourceWithRawResponse,
    ChangelogResourceWithStreamingResponse,
    AsyncChangelogResourceWithStreamingResponse,
)
from ....._types import Body, Omit, Query, Headers, NotGiven, omit, not_given
from ....._utils import path_template, maybe_transform, async_maybe_transform
from ....._compat import cached_property
from ....._resource import SyncAPIResource, AsyncAPIResource
from ....._response import (
    to_raw_response_wrapper,
    to_streamed_response_wrapper,
    async_to_raw_response_wrapper,
    async_to_streamed_response_wrapper,
)
from ....._wrappers import ResultWrapper
from .....pagination import SyncCursorPaginationAfter, AsyncCursorPaginationAfter
from ....._base_client import AsyncPaginator, make_request_options
from .....types.flagship.apps import flag_list_params, flag_create_params, flag_update_params
from .....types.flagship.apps.flag_get_response import FlagGetResponse
from .....types.flagship.apps.flag_list_response import FlagListResponse
from .....types.flagship.apps.flag_create_response import FlagCreateResponse
from .....types.flagship.apps.flag_delete_response import FlagDeleteResponse
from .....types.flagship.apps.flag_update_response import FlagUpdateResponse

__all__ = ["FlagsResource", "AsyncFlagsResource"]


class FlagsResource(SyncAPIResource):
    @cached_property
    def changelog(self) -> ChangelogResource:
        return ChangelogResource(self._client)

    @cached_property
    def with_raw_response(self) -> FlagsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return FlagsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> FlagsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return FlagsResourceWithStreamingResponse(self)

    def create(
        self,
        app_id: str,
        *,
        account_id: str,
        default_variation: str,
        enabled: bool,
        key: str,
        rules: Iterable[flag_create_params.Rule],
        variations: Dict[str, Union[Optional[str], float, bool, Dict[str, object], Iterable[object]]],
        description: Optional[str] | Omit = omit,
        type: Literal["boolean", "string", "number", "json"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FlagCreateResponse:
        """Creates a flag.

        Returns 409 if the key already exists. `type` is inferred from
        variation values and may be omitted.

        Args:
          account_id: Cloudflare account ID.

          app_id: App identifier.

          default_variation: Variation served when no rule matches or the flag is disabled. Must be a key in
              `variations`.

          enabled: When false, the flag bypasses all rules and always serves `default_variation`.

          key: Unique identifier for the flag within an app. Used in all evaluation and SDK
              calls.

          rules: Targeting rules evaluated in ascending `priority`; the first matching rule wins.
              An empty array means the flag always serves `default_variation`.

          variations: Map of variation name to value. All values must be the same type (boolean,
              string, number, or JSON object/array). Each serialized value must be 10KB or
              smaller.

          type: Value type of the flag's variations. Inferred from the variation values on
              write, so it may be omitted in requests.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        return self._post(
            path_template("/accounts/{account_id}/flagship/apps/{app_id}/flags", account_id=account_id, app_id=app_id),
            body=maybe_transform(
                {
                    "default_variation": default_variation,
                    "enabled": enabled,
                    "key": key,
                    "rules": rules,
                    "variations": variations,
                    "description": description,
                    "type": type,
                },
                flag_create_params.FlagCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[FlagCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[FlagCreateResponse], ResultWrapper[FlagCreateResponse]),
        )

    def update(
        self,
        flag_key: str,
        *,
        account_id: str,
        app_id: str,
        default_variation: str,
        enabled: bool,
        key: str,
        rules: Iterable[flag_update_params.Rule],
        variations: Dict[str, Union[Optional[str], float, bool, Dict[str, object], Iterable[object]]],
        description: Optional[str] | Omit = omit,
        type: Literal["boolean", "string", "number", "json"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FlagUpdateResponse:
        """Replaces the entire flag definition.

        Omitted fields are dropped, not preserved —
        read before writing. Each update appends a changelog entry.

        Args:
          account_id: Cloudflare account ID.

          app_id: App identifier.

          flag_key: Flag key (slug).

          default_variation: Variation served when no rule matches or the flag is disabled. Must be a key in
              `variations`.

          enabled: When false, the flag bypasses all rules and always serves `default_variation`.

          key: Unique identifier for the flag within an app. Used in all evaluation and SDK
              calls.

          rules: Targeting rules evaluated in ascending `priority`; the first matching rule wins.
              An empty array means the flag always serves `default_variation`.

          variations: Map of variation name to value. All values must be the same type (boolean,
              string, number, or JSON object/array). Each serialized value must be 10KB or
              smaller.

          type: Value type of the flag's variations. Inferred from the variation values on
              write, so it may be omitted in requests.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        if not flag_key:
            raise ValueError(f"Expected a non-empty value for `flag_key` but received {flag_key!r}")
        return self._put(
            path_template(
                "/accounts/{account_id}/flagship/apps/{app_id}/flags/{flag_key}",
                account_id=account_id,
                app_id=app_id,
                flag_key=flag_key,
            ),
            body=maybe_transform(
                {
                    "default_variation": default_variation,
                    "enabled": enabled,
                    "key": key,
                    "rules": rules,
                    "variations": variations,
                    "description": description,
                    "type": type,
                },
                flag_update_params.FlagUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[FlagUpdateResponse]._unwrapper,
            ),
            cast_to=cast(Type[FlagUpdateResponse], ResultWrapper[FlagUpdateResponse]),
        )

    def list(
        self,
        app_id: str,
        *,
        account_id: str,
        cursor: str | Omit = omit,
        limit: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> SyncCursorPaginationAfter[FlagListResponse]:
        """Lists an app's flags ordered by key.

        Pass `cursor` from `result_info` to page
        forward; a null cursor indicates the last page.

        Args:
          account_id: Cloudflare account ID.

          app_id: App identifier.

          cursor: Pagination cursor from a previous response.

          limit: Max items to return (1–200).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/flagship/apps/{app_id}/flags", account_id=account_id, app_id=app_id),
            page=SyncCursorPaginationAfter[FlagListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "limit": limit,
                    },
                    flag_list_params.FlagListParams,
                ),
            ),
            model=FlagListResponse,
        )

    def delete(
        self,
        flag_key: str,
        *,
        account_id: str,
        app_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FlagDeleteResponse:
        """Permanently deletes a flag.

        Subsequent evaluations fall back to the
        caller-supplied default. Cannot be undone.

        Args:
          account_id: Cloudflare account ID.

          app_id: App identifier.

          flag_key: Flag key (slug).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        if not flag_key:
            raise ValueError(f"Expected a non-empty value for `flag_key` but received {flag_key!r}")
        return self._delete(
            path_template(
                "/accounts/{account_id}/flagship/apps/{app_id}/flags/{flag_key}",
                account_id=account_id,
                app_id=app_id,
                flag_key=flag_key,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[FlagDeleteResponse]._unwrapper,
            ),
            cast_to=cast(Type[FlagDeleteResponse], ResultWrapper[FlagDeleteResponse]),
        )

    def get(
        self,
        flag_key: str,
        *,
        account_id: str,
        app_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FlagGetResponse:
        """
        Returns the full flag definition including rules, variations, and audit fields.

        Args:
          account_id: Cloudflare account ID.

          app_id: App identifier.

          flag_key: Flag key (slug).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        if not flag_key:
            raise ValueError(f"Expected a non-empty value for `flag_key` but received {flag_key!r}")
        return self._get(
            path_template(
                "/accounts/{account_id}/flagship/apps/{app_id}/flags/{flag_key}",
                account_id=account_id,
                app_id=app_id,
                flag_key=flag_key,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[FlagGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[FlagGetResponse], ResultWrapper[FlagGetResponse]),
        )


class AsyncFlagsResource(AsyncAPIResource):
    @cached_property
    def changelog(self) -> AsyncChangelogResource:
        return AsyncChangelogResource(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncFlagsResourceWithRawResponse:
        """
        This property can be used as a prefix for any HTTP method call to return
        the raw response object instead of the parsed content.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#accessing-raw-response-data-eg-headers
        """
        return AsyncFlagsResourceWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncFlagsResourceWithStreamingResponse:
        """
        An alternative to `.with_raw_response` that doesn't eagerly read the response body.

        For more information, see https://www.github.com/cloudflare/cloudflare-python#with_streaming_response
        """
        return AsyncFlagsResourceWithStreamingResponse(self)

    async def create(
        self,
        app_id: str,
        *,
        account_id: str,
        default_variation: str,
        enabled: bool,
        key: str,
        rules: Iterable[flag_create_params.Rule],
        variations: Dict[str, Union[Optional[str], float, bool, Dict[str, object], Iterable[object]]],
        description: Optional[str] | Omit = omit,
        type: Literal["boolean", "string", "number", "json"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FlagCreateResponse:
        """Creates a flag.

        Returns 409 if the key already exists. `type` is inferred from
        variation values and may be omitted.

        Args:
          account_id: Cloudflare account ID.

          app_id: App identifier.

          default_variation: Variation served when no rule matches or the flag is disabled. Must be a key in
              `variations`.

          enabled: When false, the flag bypasses all rules and always serves `default_variation`.

          key: Unique identifier for the flag within an app. Used in all evaluation and SDK
              calls.

          rules: Targeting rules evaluated in ascending `priority`; the first matching rule wins.
              An empty array means the flag always serves `default_variation`.

          variations: Map of variation name to value. All values must be the same type (boolean,
              string, number, or JSON object/array). Each serialized value must be 10KB or
              smaller.

          type: Value type of the flag's variations. Inferred from the variation values on
              write, so it may be omitted in requests.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        return await self._post(
            path_template("/accounts/{account_id}/flagship/apps/{app_id}/flags", account_id=account_id, app_id=app_id),
            body=await async_maybe_transform(
                {
                    "default_variation": default_variation,
                    "enabled": enabled,
                    "key": key,
                    "rules": rules,
                    "variations": variations,
                    "description": description,
                    "type": type,
                },
                flag_create_params.FlagCreateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[FlagCreateResponse]._unwrapper,
            ),
            cast_to=cast(Type[FlagCreateResponse], ResultWrapper[FlagCreateResponse]),
        )

    async def update(
        self,
        flag_key: str,
        *,
        account_id: str,
        app_id: str,
        default_variation: str,
        enabled: bool,
        key: str,
        rules: Iterable[flag_update_params.Rule],
        variations: Dict[str, Union[Optional[str], float, bool, Dict[str, object], Iterable[object]]],
        description: Optional[str] | Omit = omit,
        type: Literal["boolean", "string", "number", "json"] | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FlagUpdateResponse:
        """Replaces the entire flag definition.

        Omitted fields are dropped, not preserved —
        read before writing. Each update appends a changelog entry.

        Args:
          account_id: Cloudflare account ID.

          app_id: App identifier.

          flag_key: Flag key (slug).

          default_variation: Variation served when no rule matches or the flag is disabled. Must be a key in
              `variations`.

          enabled: When false, the flag bypasses all rules and always serves `default_variation`.

          key: Unique identifier for the flag within an app. Used in all evaluation and SDK
              calls.

          rules: Targeting rules evaluated in ascending `priority`; the first matching rule wins.
              An empty array means the flag always serves `default_variation`.

          variations: Map of variation name to value. All values must be the same type (boolean,
              string, number, or JSON object/array). Each serialized value must be 10KB or
              smaller.

          type: Value type of the flag's variations. Inferred from the variation values on
              write, so it may be omitted in requests.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        if not flag_key:
            raise ValueError(f"Expected a non-empty value for `flag_key` but received {flag_key!r}")
        return await self._put(
            path_template(
                "/accounts/{account_id}/flagship/apps/{app_id}/flags/{flag_key}",
                account_id=account_id,
                app_id=app_id,
                flag_key=flag_key,
            ),
            body=await async_maybe_transform(
                {
                    "default_variation": default_variation,
                    "enabled": enabled,
                    "key": key,
                    "rules": rules,
                    "variations": variations,
                    "description": description,
                    "type": type,
                },
                flag_update_params.FlagUpdateParams,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[FlagUpdateResponse]._unwrapper,
            ),
            cast_to=cast(Type[FlagUpdateResponse], ResultWrapper[FlagUpdateResponse]),
        )

    def list(
        self,
        app_id: str,
        *,
        account_id: str,
        cursor: str | Omit = omit,
        limit: str | Omit = omit,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> AsyncPaginator[FlagListResponse, AsyncCursorPaginationAfter[FlagListResponse]]:
        """Lists an app's flags ordered by key.

        Pass `cursor` from `result_info` to page
        forward; a null cursor indicates the last page.

        Args:
          account_id: Cloudflare account ID.

          app_id: App identifier.

          cursor: Pagination cursor from a previous response.

          limit: Max items to return (1–200).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        return self._get_api_list(
            path_template("/accounts/{account_id}/flagship/apps/{app_id}/flags", account_id=account_id, app_id=app_id),
            page=AsyncCursorPaginationAfter[FlagListResponse],
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                query=maybe_transform(
                    {
                        "cursor": cursor,
                        "limit": limit,
                    },
                    flag_list_params.FlagListParams,
                ),
            ),
            model=FlagListResponse,
        )

    async def delete(
        self,
        flag_key: str,
        *,
        account_id: str,
        app_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FlagDeleteResponse:
        """Permanently deletes a flag.

        Subsequent evaluations fall back to the
        caller-supplied default. Cannot be undone.

        Args:
          account_id: Cloudflare account ID.

          app_id: App identifier.

          flag_key: Flag key (slug).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        if not flag_key:
            raise ValueError(f"Expected a non-empty value for `flag_key` but received {flag_key!r}")
        return await self._delete(
            path_template(
                "/accounts/{account_id}/flagship/apps/{app_id}/flags/{flag_key}",
                account_id=account_id,
                app_id=app_id,
                flag_key=flag_key,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[FlagDeleteResponse]._unwrapper,
            ),
            cast_to=cast(Type[FlagDeleteResponse], ResultWrapper[FlagDeleteResponse]),
        )

    async def get(
        self,
        flag_key: str,
        *,
        account_id: str,
        app_id: str,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = not_given,
    ) -> FlagGetResponse:
        """
        Returns the full flag definition including rules, variations, and audit fields.

        Args:
          account_id: Cloudflare account ID.

          app_id: App identifier.

          flag_key: Flag key (slug).

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        if not account_id:
            raise ValueError(f"Expected a non-empty value for `account_id` but received {account_id!r}")
        if not app_id:
            raise ValueError(f"Expected a non-empty value for `app_id` but received {app_id!r}")
        if not flag_key:
            raise ValueError(f"Expected a non-empty value for `flag_key` but received {flag_key!r}")
        return await self._get(
            path_template(
                "/accounts/{account_id}/flagship/apps/{app_id}/flags/{flag_key}",
                account_id=account_id,
                app_id=app_id,
                flag_key=flag_key,
            ),
            options=make_request_options(
                extra_headers=extra_headers,
                extra_query=extra_query,
                extra_body=extra_body,
                timeout=timeout,
                post_parser=ResultWrapper[FlagGetResponse]._unwrapper,
            ),
            cast_to=cast(Type[FlagGetResponse], ResultWrapper[FlagGetResponse]),
        )


class FlagsResourceWithRawResponse:
    def __init__(self, flags: FlagsResource) -> None:
        self._flags = flags

        self.create = to_raw_response_wrapper(
            flags.create,
        )
        self.update = to_raw_response_wrapper(
            flags.update,
        )
        self.list = to_raw_response_wrapper(
            flags.list,
        )
        self.delete = to_raw_response_wrapper(
            flags.delete,
        )
        self.get = to_raw_response_wrapper(
            flags.get,
        )

    @cached_property
    def changelog(self) -> ChangelogResourceWithRawResponse:
        return ChangelogResourceWithRawResponse(self._flags.changelog)


class AsyncFlagsResourceWithRawResponse:
    def __init__(self, flags: AsyncFlagsResource) -> None:
        self._flags = flags

        self.create = async_to_raw_response_wrapper(
            flags.create,
        )
        self.update = async_to_raw_response_wrapper(
            flags.update,
        )
        self.list = async_to_raw_response_wrapper(
            flags.list,
        )
        self.delete = async_to_raw_response_wrapper(
            flags.delete,
        )
        self.get = async_to_raw_response_wrapper(
            flags.get,
        )

    @cached_property
    def changelog(self) -> AsyncChangelogResourceWithRawResponse:
        return AsyncChangelogResourceWithRawResponse(self._flags.changelog)


class FlagsResourceWithStreamingResponse:
    def __init__(self, flags: FlagsResource) -> None:
        self._flags = flags

        self.create = to_streamed_response_wrapper(
            flags.create,
        )
        self.update = to_streamed_response_wrapper(
            flags.update,
        )
        self.list = to_streamed_response_wrapper(
            flags.list,
        )
        self.delete = to_streamed_response_wrapper(
            flags.delete,
        )
        self.get = to_streamed_response_wrapper(
            flags.get,
        )

    @cached_property
    def changelog(self) -> ChangelogResourceWithStreamingResponse:
        return ChangelogResourceWithStreamingResponse(self._flags.changelog)


class AsyncFlagsResourceWithStreamingResponse:
    def __init__(self, flags: AsyncFlagsResource) -> None:
        self._flags = flags

        self.create = async_to_streamed_response_wrapper(
            flags.create,
        )
        self.update = async_to_streamed_response_wrapper(
            flags.update,
        )
        self.list = async_to_streamed_response_wrapper(
            flags.list,
        )
        self.delete = async_to_streamed_response_wrapper(
            flags.delete,
        )
        self.get = async_to_streamed_response_wrapper(
            flags.get,
        )

    @cached_property
    def changelog(self) -> AsyncChangelogResourceWithStreamingResponse:
        return AsyncChangelogResourceWithStreamingResponse(self._flags.changelog)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, Annotated, TypedDict

from ...._utils import PropertyInfo

__all__ = ["FleetStatusOverTimeParams"]


class FleetStatusOverTimeParams(TypedDict, total=False):
    account_id: Required[str]
    """Unique identifier linked to an account."""

    from_: Required[Annotated[str, PropertyInfo(alias="from")]]
    """Start of the time range to query.

    Timestamp can be provided in ISO 8601 datetime format or milliseconds since
    epoch.
    """

    to: Required[str]
    """End of the time range to query.

    Timestamp can be provided in ISO 8601 datetime format or milliseconds since
    epoch.
    """

    colo: str
    """Cloudflare colo airport code."""

    device_id: str
    """Device-specific ID, given as UUID."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Optional
from datetime import datetime

from ....._models import BaseModel

__all__ = ["ISPs", "ISP", "ISPIP", "ISPIPLocation"]


class ISPIPLocation(BaseModel):
    """Geographic location information.

    All fields are returned as the literal string `"REDACTED"` for callers that do not have the PII permission.
    """

    city: Optional[str] = None
    """City name. Returned as `"REDACTED"` without PII permission."""

    country_iso: Optional[str] = None
    """Country ISO code. Returned as `"REDACTED"` without PII permission."""

    state_iso: Optional[str] = None
    """State/province ISO code. Returned as `"REDACTED"` without PII permission."""

    zip: Optional[str] = None
    """ZIP/postal code. Returned as `"REDACTED"` without PII permission."""


class ISPIP(BaseModel):
    """IP address information for the ISP hop.

    Fields marked as PII-gated (`name`, `address`, `netmask`, and all `location` sub-fields) will be returned as the literal string `"REDACTED"` for callers that do not have the PII permission. `asn`, `aso`, and `version` are always returned regardless of PII access.
    """

    address: Optional[str] = None
    """IP address. Returned as `"REDACTED"` without PII permission."""

    asn: Optional[int] = None
    """Autonomous System Number."""

    aso: Optional[str] = None
    """Autonomous System Organization name."""

    location: Optional[ISPIPLocation] = None
    """Geographic location information.

    All fields are returned as the literal string `"REDACTED"` for callers that do
    not have the PII permission.
    """

    name: Optional[str] = None
    """Named IP address (reverse DNS hostname when available).

    Returned as `"REDACTED"` without PII permission.
    """

    netmask: Optional[str] = None
    """Network mask. Returned as `"REDACTED"` without PII permission."""

    version: Optional[int] = None
    """IP version (`1` for IPv4, `2` for IPv6, `0` if unknown)."""


class ISP(BaseModel):
    test_id: str
    """The test that generated this result."""

    test_result_id: str
    """The specific test result."""

    time_start: datetime
    """Timestamp of when the ISP was observed."""

    ip: Optional[ISPIP] = None
    """IP address information for the ISP hop.

    Fields marked as PII-gated (`name`, `address`, `netmask`, and all `location`
    sub-fields) will be returned as the literal string `"REDACTED"` for callers that
    do not have the PII permission. `asn`, `aso`, and `version` are always returned
    regardless of PII access.
    """


class ISPs(BaseModel):
    isps: List[ISP]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .jobs import (
    JobsResource,
    AsyncJobsResource,
    JobsResourceWithRawResponse,
    AsyncJobsResourceWithRawResponse,
    JobsResourceWithStreamingResponse,
    AsyncJobsResourceWithStreamingResponse,
)
from .items import (
    ItemsResource,
    AsyncItemsResource,
    ItemsResourceWithRawResponse,
    AsyncItemsResourceWithRawResponse,
    ItemsResourceWithStreamingResponse,
    AsyncItemsResourceWithStreamingResponse,
)
from .instances import (
    InstancesResource,
    AsyncInstancesResource,
    InstancesResourceWithRawResponse,
    AsyncInstancesResourceWithRawResponse,
    InstancesResourceWithStreamingResponse,
    AsyncInstancesResourceWithStreamingResponse,
)

__all__ = [
    "JobsResource",
    "AsyncJobsResource",
    "JobsResourceWithRawResponse",
    "AsyncJobsResourceWithRawResponse",
    "JobsResourceWithStreamingResponse",
    "AsyncJobsResourceWithStreamingResponse",
    "ItemsResource",
    "AsyncItemsResource",
    "ItemsResourceWithRawResponse",
    "AsyncItemsResourceWithRawResponse",
    "ItemsResourceWithStreamingResponse",
    "AsyncItemsResourceWithStreamingResponse",
    "InstancesResource",
    "AsyncInstancesResource",
    "InstancesResourceWithRawResponse",
    "AsyncInstancesResourceWithRawResponse",
    "InstancesResourceWithStreamingResponse",
    "AsyncInstancesResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ...._models import BaseModel

__all__ = ["PrefixBindingCreateResponse"]


class PrefixBindingCreateResponse(BaseModel):
    id: str
    """The ID of the binding."""

    cidr: str
    """The CIDR that is bound."""

    prefix_id: str
    """The ID of the parent prefix."""

    region_key: str
    """The region key used for the binding."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .ramps import (
    RampsResource,
    AsyncRampsResource,
    RampsResourceWithRawResponse,
    AsyncRampsResourceWithRawResponse,
    RampsResourceWithStreamingResponse,
    AsyncRampsResourceWithStreamingResponse,
)
from .cf1_sites import (
    Cf1SitesResource,
    AsyncCf1SitesResource,
    Cf1SitesResourceWithRawResponse,
    AsyncCf1SitesResourceWithRawResponse,
    Cf1SitesResourceWithStreamingResponse,
    AsyncCf1SitesResourceWithStreamingResponse,
)

__all__ = [
    "RampsResource",
    "AsyncRampsResource",
    "RampsResourceWithRawResponse",
    "AsyncRampsResourceWithRawResponse",
    "RampsResourceWithStreamingResponse",
    "AsyncRampsResourceWithStreamingResponse",
    "Cf1SitesResource",
    "AsyncCf1SitesResource",
    "Cf1SitesResourceWithRawResponse",
    "AsyncCf1SitesResourceWithRawResponse",
    "Cf1SitesResourceWithStreamingResponse",
    "AsyncCf1SitesResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

__all__ = ["RecipientListParams"]


class RecipientListParams(TypedDict, total=False):
    account_id: Required[str]
    """Account identifier."""

    include_resources: bool
    """Include resources in the response."""

    page: int
    """Page number.

    Defaults to `1` when `per_page` is supplied without `page`. May be omitted
    entirely along with `per_page` to receive a non-paginated response.
    """

    per_page: int
    """Number of objects to return per page.

    Defaults to `20` when `page` is supplied without `per_page`. May be omitted
    entirely along with `page` to receive a non-paginated response.
    """



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing_extensions import Required, TypedDict

from .site_location_param import SiteLocationParam

__all__ = ["SiteEditParams"]


class SiteEditParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier"""

    connector_id: str
    """Magic Connector identifier tag."""

    description: str

    location: SiteLocationParam
    """Location of site in latitude and longitude."""

    name: str
    """The name of the site."""

    secondary_connector_id: str
    """Magic Connector identifier tag. Used when high availability mode is on."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .data_tags import (
    DataTagsResource,
    AsyncDataTagsResource,
    DataTagsResourceWithRawResponse,
    AsyncDataTagsResourceWithRawResponse,
    DataTagsResourceWithStreamingResponse,
    AsyncDataTagsResourceWithStreamingResponse,
)
from .data_tag_categories import (
    DataTagCategoriesResource,
    AsyncDataTagCategoriesResource,
    DataTagCategoriesResourceWithRawResponse,
    AsyncDataTagCategoriesResourceWithRawResponse,
    DataTagCategoriesResourceWithStreamingResponse,
    AsyncDataTagCategoriesResourceWithStreamingResponse,
)

__all__ = [
    "DataTagsResource",
    "AsyncDataTagsResource",
    "DataTagsResourceWithRawResponse",
    "AsyncDataTagsResourceWithRawResponse",
    "DataTagsResourceWithStreamingResponse",
    "AsyncDataTagsResourceWithStreamingResponse",
    "DataTagCategoriesResource",
    "AsyncDataTagCategoriesResource",
    "DataTagCategoriesResourceWithRawResponse",
    "AsyncDataTagCategoriesResourceWithRawResponse",
    "DataTagCategoriesResourceWithStreamingResponse",
    "AsyncDataTagCategoriesResourceWithStreamingResponse",
]



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Optional
from typing_extensions import Required, TypedDict

__all__ = ["URLIgnorePatternEditParams"]


class URLIgnorePatternEditParams(TypedDict, total=False):
    account_id: Required[str]
    """Identifier."""

    comments: Optional[str]
    """Optional note describing the reason for the ignore pattern."""

    pattern: str
    """Regular expression matching URLs that should not be rewritten."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import Optional

from ..._models import BaseModel

__all__ = ["Cf1SiteLocation"]


class Cf1SiteLocation(BaseModel):
    lat: Optional[float] = None
    """Latitude of the CF1 Site."""

    long: Optional[float] = None
    """Longitude of the CF1 Site."""

    name: Optional[str] = None
    """Name of nearest town, city, or village."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from typing_extensions import Literal, TypeAlias

from pydantic import Field as FieldInfo

from ...._models import BaseModel

__all__ = [
    "QueryListResponse",
    "Parameters",
    "ParametersCalculation",
    "ParametersFilter",
    "ParametersFilterUnionMember0",
    "ParametersFilterWorkersObservabilityFilterLeaf",
    "ParametersGroupBy",
    "ParametersHaving",
    "ParametersNeedle",
    "ParametersOrderBy",
]


class ParametersCalculation(BaseModel):
    operator: Literal[
        "uniq",
        "count",
        "max",
        "min",
        "sum",
        "avg",
        "median",
        "p001",
        "p01",
        "p05",
        "p10",
        "p25",
        "p75",
        "p90",
        "p95",
        "p99",
        "p999",
        "stddev",
        "variance",
        "COUNT_DISTINCT",
        "COUNT",
        "MAX",
        "MIN",
        "SUM",
        "AVG",
        "MEDIAN",
        "P001",
        "P01",
        "P05",
        "P10",
        "P25",
        "P75",
        "P90",
        "P95",
        "P99",
        "P999",
        "STDDEV",
        "VARIANCE",
    ]

    alias: Optional[str] = None

    key: Optional[str] = None

    key_type: Optional[Literal["string", "number", "boolean"]] = FieldInfo(alias="keyType", default=None)


class ParametersFilterUnionMember0(BaseModel):
    filter_combination: Literal["and", "or", "AND", "OR"] = FieldInfo(alias="filterCombination")

    filters: List[object]

    kind: Literal["group"]


class ParametersFilterWorkersObservabilityFilterLeaf(BaseModel):
    """A filter condition applied to query results.

    Use the keys and values endpoints to discover available fields and their values before constructing filters.
    """

    key: str
    """Filter field name.

    Use verified keys from previous query results or the keys endpoint. Common keys
    include $metadata.service, $metadata.origin, $metadata.trigger,
    $metadata.message, and $metadata.error.
    """

    operation: Literal[
        "includes",
        "not_includes",
        "starts_with",
        "ends_with",
        "regex",
        "exists",
        "is_null",
        "in",
        "not_in",
        "eq",
        "neq",
        "gt",
        "gte",
        "lt",
        "lte",
        "=",
        "!=",
        ">",
        ">=",
        "<",
        "<=",
        "INCLUDES",
        "DOES_NOT_INCLUDE",
        "MATCH_REGEX",
        "EXISTS",
        "DOES_NOT_EXIST",
        "IN",
        "NOT_IN",
        "STARTS_WITH",
        "ENDS_WITH",
    ]
    """Comparison operator.

    String operators: includes, not_includes, starts_with, ends_with, regex.
    Existence: exists, is_null. Set membership: in, not_in (comma-separated values).
    Numeric: eq, neq, gt, gte, lt, lte.
    """

    type: Literal["string", "number", "boolean"]
    """Data type of the filter field.

    Must match the actual type of the key being filtered.
    """

    kind: Optional[Literal["filter"]] = None
    """Discriminator for leaf filter nodes.

    Always 'filter' when present; may be omitted.
    """

    value: Union[str, float, bool, None] = None
    """Comparison value.

    Must match actual values in your data — verify with the values endpoint. Ensure
    the value type (string/number/boolean) matches the field type. String
    comparisons are case-sensitive. Regex uses RE2 syntax (no
    lookaheads/lookbehinds).
    """


ParametersFilter: TypeAlias = Union[ParametersFilterUnionMember0, ParametersFilterWorkersObservabilityFilterLeaf]


class ParametersGroupBy(BaseModel):
    type: Literal["string", "number", "boolean"]

    value: str


class ParametersHaving(BaseModel):
    key: str

    operation: Literal["eq", "neq", "gt", "gte", "lt", "lte"]

    value: float


class ParametersNeedle(BaseModel):
    """Define an expression to search using full-text search."""

    value: Union[str, float, bool]

    is_regex: Optional[bool] = FieldInfo(alias="isRegex", default=None)

    match_case: Optional[bool] = FieldInfo(alias="matchCase", default=None)


class ParametersOrderBy(BaseModel):
    """Configure the order of the results returned by the query."""

    value: str
    """Configure which Calculation to order the results by."""

    order: Optional[Literal["asc", "desc"]] = None
    """Set the order of the results"""


class Parameters(BaseModel):
    calculations: Optional[List[ParametersCalculation]] = None
    """Create Calculations to compute as part of the query."""

    datasets: Optional[List[str]] = None
    """Set the Datasets to query. Leave it empty to query all the datasets."""

    filter_combination: Optional[Literal["and", "or", "AND", "OR"]] = FieldInfo(alias="filterCombination", default=None)
    """Set a Flag to describe how to combine the filters on the query."""

    filters: Optional[List[ParametersFilter]] = None
    """Configure the Filters to apply to the query.

    Supports nested groups via kind: 'group'.
    """

    group_bys: Optional[List[ParametersGroupBy]] = FieldInfo(alias="groupBys", default=None)
    """Define how to group the results of the query."""

    havings: Optional[List[ParametersHaving]] = None
    """Configure the Having clauses that filter on calculations in the query result."""

    limit: Optional[int] = None
    """Set a limit on the number of results / records returned by the query"""

    needle: Optional[ParametersNeedle] = None
    """Define an expression to search using full-text search."""

    order_by: Optional[ParametersOrderBy] = FieldInfo(alias="orderBy", default=None)
    """Configure the order of the results returned by the query."""


class QueryListResponse(BaseModel):
    id: str

    adhoc: bool
    """If the query wasn't explcitly saved"""

    created: str

    created_by: str = FieldInfo(alias="createdBy")

    description: Optional[str] = None

    name: str
    """Query name"""

    parameters: Parameters

    updated: str

    updated_by: str = FieldInfo(alias="updatedBy")



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from ...._models import BaseModel

__all__ = ["RelateDeleteResponse"]


class RelateDeleteResponse(BaseModel):
    success: bool
